import sys
import os
import torch
import tqdm
import fire
from opendr.engine.target import BoundingBox3D
from opendr.engine.datasets import PointCloudsDatasetIterator
from opendr.perception.object_detection_3d import VoxelObjectDetection3DLearner
from opendr.perception.object_detection_3d import (
    LabeledPointCloudsDatasetIterator,
)
from opendr.perception.object_tracking_3d.datasets.kitti_tracking import LabeledTrackingPointCloudsDatasetIterator

config_roots = {
    "pointpillars": os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "voxel_object_detection_3d",
        "second_detector",
        "configs",
        "pointpillars",
        "car",
    ),
    "tanet": os.path.join(
        ".",
        "src",
        "opendr",
        "perception",
        "object_detection_3d",
        "voxel_object_detection_3d",
        "second_detector",
        "configs",
        "tanet",
        "car",
    ),
}
temp_dir = "./run/models"
subsets_path = os.path.join(
    ".",
    "src",
    "opendr",
    "perception",
    "object_detection_3d",
    "datasets",
    "kitti_subsets",
)

dataset_tracking_path = "/data/sets/kitti_tracking"
datasets = {}

all_track_ids = ["0000"]

for track_id in all_track_ids:
    datasets[track_id] = LabeledTrackingPointCloudsDatasetIterator(
        dataset_tracking_path + "/training/velodyne/" + track_id,
        dataset_tracking_path + "/training/label_02/" + track_id + ".txt",
        dataset_tracking_path + "/training/calib/" + track_id + ".txt",
    )


def save_detection_inference(
    model_type,
    device="cuda:0",
    name="pointpillars_car",
    samples_list=[1],
    config="xyres_16.proto",
    eval_suffix="classic",
):

    config = os.path.join(config_roots[model_type], config,)
    model_path = os.path.join(temp_dir, name)

    learner = VoxelObjectDetection3DLearner(
        model_config_path=config, device=device, checkpoint_after_iter=1000, return_uncertainty=True
    )
    learner.load(model_path)

    results = {}
    for samples in samples_list:
        print("samples =", samples)

        for track_id in all_track_ids:

            dataset = datasets[track_id]
            
            os.makedirs(os.path.join(
                model_path, "tracking_inference_detections", eval_suffix, "samples_" + str(samples)
            ), exist_ok=True)

            with open(
                os.path.join(
                    model_path,
                    "tracking_inference_detections",
                    eval_suffix,
                    "samples_" + str(samples),
                    track_id + ".txt"
                ), "w"
            ) as f:
                for frame, (input, _) in tqdm.tqdm(enumerate(dataset), total=len(dataset)):
                    output = learner.infer(input, samples=samples)

                    result = "\n".join(box.to_kitti_tracking_string(frame) for box in output)

                    if len(output) > 0:
                        result += "\n"

                    f.write(result)



def test_model(
    model_type,
    device="cuda:0",
    name="pointpillars_car",
    samples_list=[1],
    config="xyres_16.proto",
    eval_suffix="classic",
):

    return save_detection_inference(
        model_type=model_type,
        device=device,
        name=name,
        samples_list=samples_list,
        config=config,
        eval_suffix=eval_suffix,
    )

    # config = os.path.join(config_roots[model_type], config,)
    # model_path = os.path.join(temp_dir, name)

    # learner = VoxelObjectDetection3DLearner(
    #     model_config_path=config, device=device, checkpoint_after_iter=1000
    # )
    # learner.load(model_path)



def test_pointpillars(
    device="cuda:0",
    name="pointpillars_car",
    samples_list=[1],
    config="xyres_16.proto",
    eval_suffix="classic",
):
    return test_model(
        "pointpillars",
        device=device,
        name=name,
        samples_list=samples_list,
        config=config,
        eval_suffix=eval_suffix,
    )


def test_tanet(
    device="cuda:0",
    name="tanet_car",
    samples_list=[1],
    config="xyres_16.proto",
    eval_suffix="classic",
):
    return test_model(
        "tanet",
        device=device,
        name=name,
        samples_list=samples_list,
        config=config,
        eval_suffix=eval_suffix,
    )


def test_vnn_pointpillars(
    device="cuda:0",
    name="vnn_pointpillars_car",
    samples_list=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
    config="vnn_xyres_16.proto",
    eval_suffix="vnn",
):
    return test_pointpillars(
        device=device,
        name=name,
        samples_list=samples_list,
        config=config,
        eval_suffix=eval_suffix,
    )

def test_vnna_pointpillars(
    device="cuda:0",
    name="vnna_pointpillars_car",
    samples_list=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
    config="vnna_xyres_16.proto",
    eval_suffix="vnn",
):
    return test_pointpillars(
        device=device,
        name=name,
        samples_list=samples_list,
        config=config,
        eval_suffix=eval_suffix,
    )


def test_vnn_tanet(
    device="cuda:0",
    name="vnn_tanet_car",
    samples_list=[1, 2, 3, 4],
    config="vnn_xyres_16.proto",
    eval_suffix="vnn",
):
    return test_model(
        "tanet",
        device=device,
        name=name,
        samples_list=samples_list,
        config=config,
        eval_suffix=eval_suffix,
    )


if __name__ == "__main__":
    fire.Fire()
